
/*  @(#)toolfuns.c 1.13 95/08/25
 *
 *  Various routines used by the mptool program.
 *
 *  Copyright (c) Rich Burridge.    All rights reserved.
 *
 *  Permission is given to distribute these sources, as long as the
 *  copyright messages are not removed, and no monies are exchanged.
 *
 *  No responsibility is taken for any errors or inaccuracies inherent
 *  either to the comments or the code of this program, but if
 *  reported to me then an attempt will be made to fix them.
 */

#include "mptool.h"
#include <ctype.h>
#include <sys/param.h>
#include <sys/stat.h>
#include "tu_runtime.h"

static void exec_print_job  (char *) ;


void
do_print_cmd(char *filenames)
{
  char *aliasname, buf[BUFFERSIZE], *gsubject, m[MAXLINE], *printer_name ;
  int i ;

  int copies = get_int_value(F_MAIN, WI_COPIES) ;

  set_icon(I_PRINTING) ;

#ifdef DEBUG
  STRCPY(buf, "echo mp ") ;
#else
  STRCPY(buf, "mp ") ;
#endif // DEBUG

  buf_arg(buf, "-printer") ;                        /* -printer */
  printer_name = str_opt_val(O_PR_NAME) ;
  if (printer_name && *printer_name != '\0')
    buf_str(buf, printer_name) ;

  if (get_bool_value(F_MAIN, WI_FILE_TYPE_NEWS))    /* -article */
    buf_arg(buf, "-article") ;
  if (get_bool_value(F_MAIN, WI_FILE_TYPE_POST))    /* -postscript */
    buf_arg(buf, "-postscript") ;
  if (get_bool_value(F_MAIN, WI_FILE_TYPE_TEXT))    /* -text */
    buf_arg(buf, "-text") ;

  if (get_bool_value(F_MAIN, WI_OUTPUT_PERORG))
    {
      if (get_bool_value(F_MAIN, WI_ORG_TYPE_FP))   /* -franklin */
        buf_arg(buf, "-franklin") ;
      if (get_bool_value(F_MAIN, WI_ORG_TYPE_FF))   /* -filofax */
        buf_arg(buf, "-filofax") ;
      if (get_bool_value(F_MAIN, WI_ORG_TYPE_PP))   /* -proplan */
        buf_arg(buf, "-proplan") ;
      if (get_bool_value(F_MAIN, WI_ORG_TYPE_TM))   /* -timeman */
        buf_arg(buf, "-timeman") ;
      if (get_bool_value(F_MAIN, WI_ORG_TYPE_TSI))  /* -tmintl */
        buf_arg(buf, "-tsintl") ;
      if (get_bool_value(F_MAIN, WI_ORG_TYPE_TSP))  /* -tmpartner */
        buf_arg(buf, "-tspartner") ;
    }

  if (get_bool_value(F_MAIN, WI_OUTPUT_LAND))       /* -landscape */
    buf_arg(buf, "-landscape") ;

  if (get_bool_value(F_MAIN, WI_PAPER_SIZE_A4))     /* -a4 */
    buf_arg(buf, "-a4") ;
  if (get_bool_value(F_MAIN, WI_PAPER_SIZE_US))     /* -us */
    buf_arg(buf, "-us") ;

  if (!int_opt_val(O_BURSTPAGE))                    /* -noburstpage */
    buf_arg(buf, "-noburstpage") ;
  if (int_opt_val(O_FOLDER))                        /* -folder */
    buf_arg(buf, "-folder") ;
  if (int_opt_val(O_DIGEST))                        /* -digest */
    buf_arg(buf, "-digest") ;

  gsubject = get_str_value(F_MAIN, WI_SUBJECT) ;
  if (gsubject && *gsubject != '\0')                /* -subject text */
    {
      buf_arg(buf, "-subject") ;
      buf_str(buf, gsubject) ;
    }

  if (int_opt_val(O_LLEN) < 1 ||
      int_opt_val(O_LLEN) > MAXLINELENGTH)          /* -linelength # */
    {
      SPRINTF(m, mptool_strs[(int) S_INVLINE], LINELENGTH) ;
      notice(warr[WI_MAIN_FRAME], m) ;
      set_int_opt_val(O_LLEN, LINELENGTH) ;
      if (owarr)
        set_int_value(F_OUTPUT, WI_LINE_LENGTH, LINELENGTH) ;
    }
  buf_int(buf, "-linelength", int_opt_val(O_LLEN)) ;

  if (int_opt_val(O_PLEN) < 1 ||
      int_opt_val(O_PLEN) > MAXPAGELENGTH)        /* -pagelength # */
    {
      SPRINTF(m, mptool_strs[(int) S_INVPAGE], PAGELENGTH) ;
      notice(warr[WI_MAIN_FRAME], m) ;
      set_int_opt_val(O_PLEN, PAGELENGTH) ;
      if (owarr)
        set_int_value(F_OUTPUT, WI_PAGE_LENGTH, PAGELENGTH) ;
    }
  buf_int(buf, "-pagelength", int_opt_val(O_PLEN)) ;

  if (int_opt_val(O_ALLHDRS))                     /* -allhdrs */
    buf_arg(buf, "-allhdrs") ;
  else
    {
      for (i = 0; i < int_opt_val(O_REMHDRN); i++)  /* -remhdr <hdr> */
        {
          buf_arg(buf, "-remhdr") ;
          buf_str(buf, get_hdr(H_REMOVE, i)) ;
        }

      for (i = 0; i < int_opt_val(O_ADDHDRN); i++)  /* -addhdr <hdr> */
        {
          buf_arg(buf, "-addhdr") ;
          buf_str(buf, get_hdr(H_ADD, i)) ;
        }
    }

  if (copies < 1 || copies > MAXCOPIES)             /* -copies # */
    {
      SPRINTF(m, mptool_strs[(int) S_INVCOPIES], DEFCOPIES) ;
      notice(warr[WI_MAIN_FRAME], m) ;
      copies = DEFCOPIES ;
      set_int_value(F_MAIN, WI_COPIES, 1) ;
    }
  buf_int(buf, "-copies", copies) ;

  if (get_bool_value(F_MAIN, WI_NO_SIDES_DOUBLE))
    buf_arg(buf, "-twosided") ;                     /* -twosided */
  else
    buf_arg(buf, "-onesided") ;                     /* -onesided */

  aliasname = str_opt_val(O_ALIAS) ;
  if (aliasname && *aliasname != '\0')
    {
      buf_arg(buf, "-alias") ;
      buf_str(buf, aliasname) ;
    }

  if (proname && *proname != '\0')          /* -prologue name */
    {
      buf_arg(buf, "-prologue") ;
      buf_str(buf, proname) ;
    }

  buf_int(buf, "-columns", int_opt_val(O_NUMCOLS)) ;      /* -columns # */
  buf_int(buf, "-tab",     int_opt_val(O_TABSIZE)) ;      /* -tab # */
  buf_int(buf, "-chars",   int_opt_val(O_NAMELEN)) ;      /* -chars # */
  buf_int(buf, "-words",   int_opt_val(O_NAMEF)) ;        /* -words # */
  if (int_opt_val(O_WRAPW)) buf_arg(buf, "-wrap") ;       /* -wrap */
  if (int_opt_val(O_FLIP))  buf_arg(buf, "-flip") ;       /* -flip */
  if (int_opt_val(O_FORCEPAGE))                           /* -forcepage */
    buf_arg(buf, "-forcepage") ;
  if (int_opt_val(O_ELM))      buf_arg(buf, "-elm") ;       /* -elm */
  if (int_opt_val(O_NUMBER))   buf_arg(buf, "-number") ;    /* -number */
  if (int_opt_val(O_PR_ORIG))  buf_arg(buf, "-from") ;      /* -from */
  if (int_opt_val(O_CONTENT))  buf_arg(buf, "-content") ;   /* -content */
  if (int_opt_val(O_MODTIME))  buf_arg(buf, "-modtime") ;   /* -modtime */
  if (int_opt_val(O_BSDPRINT)) buf_arg(buf, "-bsdprint") ;  /* -bsdprint */
  if (int_opt_val(O_TUMBLE))   buf_arg(buf, "-tumble") ;    /* -tumble */

  buf_int(buf, "-bottom", get_margin(M_BOTTOM)) ;           /* -bottom # */
  buf_int(buf, "-left",   get_margin(M_LEFT)) ;             /* -left   # */
  buf_int(buf, "-right",  get_margin(M_RIGHT)) ;            /* -right  # */
  buf_int(buf, "-top",    get_margin(M_TOP)) ;              /* -top    # */

  if (filenames == NULL || *filenames == '\0')
    {
      notice(warr[WI_MAIN_FRAME], mptool_strs[(int) S_NOFILES]) ;
      return ;
    }
  buf_arg(buf, filenames) ;
  exec_print_job(buf) ;
  set_icon(I_NORMAL) ;
}


void
do_startup(void)
{
  char **argv, files[BUFFERSIZE], title[MAXLINE] ;
  int argc, i ;

  argc = tu_get_cmd_argc() ;
  argv = (char **) calloc(argc, sizeof(char *)) ;
  for (i = 0; i < argc; i++)
    argv[i] = tu_get_cmd_argv(i) ;

  Base() ;
  Header() ;
  Help() ;
  Input() ;
  Option() ;
  get_appname(argc, argv) ;
  Output() ;
  Print() ;
  User() ;
  Xutil() ;

  init_headers() ;           /* Initialise standard headers. */
  init_text() ;              /* Internationalise text strings. */
  read_resources() ;         /* Read resources from merged database. */
  get_options(argc, argv) ;  /* Read and process command line options */
  do_show_base() ;           /* Create base window. */

  files[0] = '\0' ;                   /* Prime the list of files to print. */
  for (i = 0; i < int_opt_val(O_NUMNAMES); i++)
    {
      STRCAT(files, get_filename(i)) ;
      STRCAT(files, " ") ;
    }
  files[strlen(files)-1] = '\0' ;
  read_str(&filenames, files) ;

  do_reset_options() ;
  XtRealizeWidget(warr[WI_MAIN_FRAME]) ;
  XmProcessTraversal(warr[WI_FILENAME], XmTRAVERSE_CURRENT) ;
  XtMapWidget(warr[WI_MAIN_FRAME]) ;
}


static void
exec_print_job(char *buf)
{
  int pid ;

  if ((pid = (int) fork()) < 0)
    {
      notice(warr[WI_MAIN_FRAME], mptool_strs[(int) S_START]) ;
      return ;
    }

  if (pid == 0)           /* Child process. */
    {
      (void) execl("/bin/sh", "sh", "-c", buf, (char *) 0) ;
      perror(mptool_strs[(int) S_EXECUTE]) ;
      _exit(1) ;
    }
}


Widget
get_widget(enum frame_type ftype, int index)
{
       if (ftype == F_MAIN   && warr)  return(warr[index]) ;
  else if (ftype == F_HDRS   && hwarr) return(hwarr[index]) ;
  else if (ftype == F_INPUT  && iwarr) return(iwarr[index]) ;
  else if (ftype == F_OUTPUT && owarr) return(owarr[index]) ;
  else if (ftype == F_PRINT  && pwarr) return(pwarr[index]) ;
  else if (ftype == F_USER   && uwarr) return(uwarr[index]) ;
  else return(NULL) ;                         /* Shouldn't happen. */
}
